@@ -4,14 +4,13 @@ import re
 import smtplib
 import socket
 import ssl
-
 from email.utils import formataddr
 
-from dns.resolver import Resolver, NXDOMAIN
+from dns.resolver import NXDOMAIN, Resolver, Cache
+from dns.exception import Timeout
 
 from .exceptions import SpoolError
 
-
 LOG = logging.getLogger(__name__)
 
 
@@ -29,14 +28,15 @@ class RemoteNotFoundError(MailerError):
     """Remote server could not be evaluated."""
 
 
+class ResolverTimeoutError(MailerError):
+    """Resolver query timed out."""
+
+
 class Mailer:
-    """
-    Represents an SMTP connection.
-    """
+    """Represents an SMTP connection."""
 
-
-    def __init__(self, relay=None, port=25, helo=None, timeout=5,
-                 starttls=False, nameservers=None, debug=False):
+    def __init__(self, relay=None, port=25, helo=None, timeout=5, debug=False,
+                 starttls=False, nameservers=None, no_cache=False):
 
         self.port = port
         self.relay = relay
@@ -48,8 +48,21 @@ class Mailer:
 
         self.timeout = timeout
         self.starttls = starttls
-        self.nameservers = nameservers
         self.debug = debug
+        self.no_cache = no_cache
+
+        if nameservers:
+            self.resolver = Resolver(configure=False)
+            self.resolver.nameservers = [
+                n.strip() for n in nameservers.split(',')
+            ]
+
+        else:
+            self.resolver = Resolver()
+
+        if not self.no_cache:
+            self.resolver.cache = Cache()
+
 
         # TODO: Add option to parser
         self.reorder_recipients = True
@@ -60,11 +73,17 @@ class Mailer:
     def __exit__(self, exc_type, exc_value, traceback):
         pass
 
-    def send(self, msg, print_only=True):
-        """Send a message"""
+    def send(self, msg, print_only=False):
+        """Send a message.
+
+        Args:
+            msg: The message to send (or print to console)
+            print_only (:obj: `bool`, optional): Whether to print the
+                message to console instead of sending to remote.
+        """
 
         if print_only:
-            return self._print(msg)
+            return self.dump(msg)
 
         sender = formataddr(msg.sender)
 
@@ -84,7 +103,7 @@ class Mailer:
             for domain, recipients in itertools.groupby(recipients, domain):
 
                 try:
-                    host = self._get_remote(domain, self.nameservers)
+                    host = self.get_remote(domain)
 
                 except RemoteNotFoundError as err:
                     LOG.error(
@@ -95,7 +114,7 @@ class Mailer:
 
     @staticmethod
     def get_helo_name():
-        """Retrive the helo/ehlo name based on the hostname"""
+        """Retrive the helo/ehlo name based on the hostname."""
 
         fqdn = socket.getfqdn()
         if '.' in fqdn:
@@ -110,36 +129,58 @@ class Mailer:
 
         return f'[{address}]'
 
-    @staticmethod
-    def _get_remote(domain, nameservers=None):
+    def get_remote(self, domain, nameservers=None, lifetime=10.0):
+        """Returns the mail exchange server for a given domain.
+
+        Args:
+            domain (str): Domain name to retrieve MX records from.
+            nameservers (:obj: `list`, optional): List of nameservers (`str`)
+                to query against.
+            lifetime (:obj: `float`, optional): Number of seconds (`float`)
+                until the query times out.
+
+        Returns:
+            str: Hostname or address of MX record with the highest preference
+                (lowest priority value).
+
+        Raises:
+            RemoteNotFoundError: If no MX resource record was found.
+            ResolverTimeoutError: DNS operation timed out.
+        """
 
         match = DOMAIN_LITERAL.fullmatch(domain)
         if match:
             return match.group('ip_address')
 
-        if nameservers:
-            resolver = Resolver(configure=False)
-            resolver.nameservers = [n.strip() for n in nameservers.split(',')]
-
-        else:
-            resolver = Resolver()
-
         try:
-            answers = resolver.query(domain, 'MX')
-            mx = min(answers, key=lambda rdata: rdata.preference).exchange
-            return mx.to_text().rstrip('.')
+            answers = self.resolver.query(domain, 'MX', lifetime=lifetime)
+            peer = min(answers, key=lambda rdata: rdata.preference).exchange
+            return peer.to_text().rstrip('.') or peer.to_text()
+
+        except Timeout:
+            raise ResolverTimeoutError('Query for mx record timed out. '
+                f'[domain={domain}, timeout={lifetime}s]')
 
         except NXDOMAIN:
             raise RemoteNotFoundError(
-                    f'No mx record found for domain: {domain}')
+                f'No mx record found for domain. [domain={domain}]')
 
     def _connect(self, host, port):
 
         LOG.info('Connecting to remote server. [host=%s, port=%s, helo=%s]',
                  host, port, self.helo)
 
-        server = smtplib.SMTP(host, port, timeout=self.timeout,
-                              local_hostname=self.helo)
+        try:
+            server = smtplib.SMTP(host, port, timeout=self.timeout,
+                                  local_hostname=self.helo)
+
+        except ConnectionRefusedError:
+            raise MailerError(
+                f'Remote refused connection. [host={host}, port={port}]')
+
+        except socket.timeout:
+            raise MailerError(
+                f'Timeout while connecting to remote. [host={host}, port={port}]')
 
         if self.debug:
             server.set_debuglevel(2)
@@ -164,22 +205,42 @@ class Mailer:
             refused = connection.sendmail(sender, recipients, msg.as_string())
 
         except smtplib.SMTPResponseException as err:
-            LOG.error(('Error while sending message: %s - %s '
-                       '[name=%s, host=%s, port=%s]'), err.smtp_code,
-                      err.smtp_error.decode(), msg.name, host, self.port)
+
+            if isinstance(err, smtplib.SMTPSenderRefused):
+                LOG.error(('Failed to send message: Sender rejected.'
+                           '[name=%s, host=%s, port=%s]'),
+                          msg.name, host, self.port)
+            else:
+                LOG.error(('Error while sending message: %s - %s '
+                           '[name=%s, host=%s, port=%s]'), err.smtp_code,
+                          err.smtp_error.decode(), msg.name, host, self.port)
 
         except smtplib.SMTPException as err:
+
+            if isinstance(err, smtplib.SMTPRecipientsRefused):
+                err = 'Remote refused all recipients.'
+            elif isinstance(err, smtplib.SMTPServerDisconnected):
+                err = 'Connection closed by remote host.'
+
             LOG.error('Failed to send message: %s [name=%s, host=%s, port=%s]',
                       err, msg.name, host, self.port)
 
         else:
-            for rcpt, (code, resp) in refused.items():
+            for recipient, (code, response) in refused.items():
                 LOG.warning('Remote refused recipient: %s [host=%s, port=%s]',
-                            rcpt, host, self.port)
+                            recipient, host, self.port)
 
             LOG.info('Message sent. [name=%s, host=%s, port=%s]',
                      msg.name, host, self.port)
 
     @staticmethod
-    def _print(msg):
+    def dump(msg):
+        """Print a message to console.
+
+        Prints a given message to console in Internet Message Format (IMF).
+
+        Args:
+            msg: A message.
+        """
+
         print(MAIL_OUT_PREFIX, msg.as_string(), MAIL_OUT_SUFFIX, sep='\n')
