@@ -2,17 +2,19 @@ import argparse
 import logging
 import sys
 import time
-
+import random
+import string
 from pathlib import Path
 
-from .mailer import Mailer, MailerError
+from .exceptions import SpoolError
+from .mailer import Mailer
 from .message import Message, MessageError
 from .parser import Config, ConfigError
-from .exceptions import SpoolError
 
 
-LOG = logging.getLogger(__name__)
 LOG_FORMAT = '%(asctime)s %(levelname)s %(message)s'
+LOG = logging.getLogger(__name__)
+
 
 
 def parse_args(args):
@@ -36,6 +38,12 @@ def parse_args(args):
         help='nameservers for lookup of MX records'
     )
 
+    parser.add_argument(
+        '-N', '--no-cache',
+        action='store_true',
+        help='disable dns cache'
+    )
+
     parser.add_argument(
         '-d', '--delay',
         type=float,
@@ -65,6 +73,11 @@ def parse_args(args):
         help='check config files and quit',
     )
 
+    parser.add_argument(
+        '-t', '--tags',
+        help='tags to execute',
+    )
+
     parser.add_argument(
         '--starttls',
         action='store_true',
@@ -94,11 +107,83 @@ def parse_args(args):
     return parser.parse_args(args)
 
 
+def tags_matches_mail(tags, mail):
+    """Returns True if mail has a matching tag."""
+
+    if not tags:
+        return True
+
+    tags = [tag.strip() for tag in tags.split(',')]
+
+    return any(tag in mail for tag in tags)
+
+
+def parse_files(path, mail):
+    copy_properties = [
+        ('from_key_file', 'from_key'),
+        ('from_crt_file', 'from_crt'),
+        ('to_crts_file', 'to_crts'),
+    ]
+
+    for src, dst in copy_properties:
+
+        if 'smime' not in mail or src not in mail['smime']:
+            continue
+
+        with open(path.parent / mail['smime'][src], 'r') as fh:
+            mail['smime'][dst] = fh.read()
+
+        del mail['smime'][src]
+
+    return mail
+
+
+class LogFormatter(logging.Formatter):
+
+    GREY = '\x1b[38;21m'
+    YELLOW = '\x1b[33;21m'
+    RED = '\x1b[31;21m'
+    BOLD_RED = '\x1b[31;1m'
+    RESET = '\x1b[0m'
+
+    FORMATS = {
+        logging.DEBUG: GREY + LOG_FORMAT + RESET,
+        logging.INFO: GREY + LOG_FORMAT + RESET,
+        logging.WARNING: YELLOW + LOG_FORMAT + RESET,
+        logging.ERROR: RED + LOG_FORMAT + RESET,
+        logging.CRITICAL: BOLD_RED + LOG_FORMAT + RESET,
+    }
+
+    def __init__(self, *args, **kwargs):
+        self.has_color = hasattr(sys.stdout, 'isatty') and sys.stdout.isatty()
+        super().__init__(*args, **kwargs)
+
+
+    def format(self, record):
+
+        if self.has_color:
+            log_fmt = self.FORMATS[record.levelno]
+        else:
+            log_fmt = LOG_FORMAT
+
+        formatter = logging.Formatter(log_fmt)
+
+        return formatter.format(record)
+
+
 def config_logger(verbosity):
     verbosity = min(verbosity, 2)
     log_level = logging.WARNING - verbosity * 10
 
-    logging.basicConfig(level=log_level, format=LOG_FORMAT)
+    console = logging.StreamHandler()
+    console.setFormatter(LogFormatter())
+
+    logging.basicConfig(level=log_level, handlers=[console,])
+
+
+def get_uuid(length=6):
+    uuid = random.sample(string.digits + string.ascii_lowercase, length)
+    return ''.join(uuid)
 
 
 def run():
@@ -127,10 +212,15 @@ def run():
 
         with Mailer(relay=args.relay, port=args.port, helo=args.helo,
                     debug=args.debug, nameservers=args.nameservers,
-                    starttls=args.starttls) as mailer:
+                    starttls=args.starttls, no_cache=args.no_cache) as mailer:
 
             for mail in config.mails:
 
+                if not tags_matches_mail(args.tags, mail.pop('tags', [])):
+                    LOG.debug('Skipping message "%s", does not match tags: %s',
+                              mail['name'], args.tags)
+                    continue
+
                 if not first and args.delay:
                     LOG.debug('Delay sending of next message by %.2f seconds.',
                               args.delay)
@@ -139,19 +229,16 @@ def run():
                     first = False
 
                 mail.pop('description', None)
+                mail = parse_files(path, mail)
                 attachments = mail.pop('attachments', [])
 
-                for prop in ('from_key', 'from_crt'):
-                    if prop in mail:
-                        mail[prop] = path.parent / mail[prop]
-
                 msg = Message(**mail)
 
                 if isinstance(attachments, str):
                     attachments = [attachments]
 
-                for a in attachments:
-                    file_path = path.parent / a
+                for attachment in attachments:
+                    file_path = path.parent / attachment
                     msg.attach(file_path)
 
                 try:
@@ -160,7 +247,7 @@ def run():
                 except MessageError as ex:
                     LOG.error(
                         'Failed to create message: %s. [name=%s, path=%s]',
-                        ex, mail.name, path
+                        ex, mail['name'], path
                     )
 
 
